#pragma once

#include "Utility.hpp"
#include "ctll/fixed_string.hpp"
#include <cpr/cpr.h>
#include <ctll.hpp>
#include <ctre.hpp>
#include <glaze/glaze.hpp>
#include <print>
#include <stdexcept>
#include <string>

struct Data {
  double cals{};
  double proteins{};
  double carbs{};
  double fats{};
};

class WolframAlpha {
public:
  WolframAlpha(const std::string& query, double amount) : m_plaintext{initPlaintext(query)}, m_amount{amount} {
    initData();
  }

  void print() {
    std::println("Cals:    {:g} kcal\nProtein: {:g} g\nCarbs:   {:g} g\nFats:    {:g} g", m_data.cals, m_data.proteins, m_data.carbs, m_data.fats);
  }

private:
  void initData() {
    const auto convertToGrams = [this](const double& match, const std::string& unit) {
      if (unit == "g") {
        return match * m_amount;
      } else if (unit == "mg") {
        return (match / 1000) * m_amount;
      } else {
        throw std::invalid_argument("Unsupported unit: " + unit);
      }
    };

    constexpr ctll::fixed_string calories_regex{R"((?<=total calories\s)\d+)"};
    constexpr ctll::fixed_string proteins_regex{R"((?<=protein\s)(\d+)\s(\w+))"};
    constexpr ctll::fixed_string carbs_regex{R"((?<=total carbohydrates\s)(\d+)\s(\w+))"};
    constexpr ctll::fixed_string fats_regex{R"((?<=total fat\s)(\d+)\s(\w+))"};

    if (auto match = ctre::search<proteins_regex>(m_plaintext); match) {
      m_data.proteins = convertToGrams(match.get<1>().to_number(), match.get<2>().to_string());
    }

    if (auto match = ctre::search<carbs_regex>(m_plaintext); match) {
      m_data.carbs = convertToGrams(match.get<1>().to_number(), match.get<2>().to_string());
    }

    if (auto match = ctre::search<fats_regex>(m_plaintext); match) {
      m_data.fats = convertToGrams(match.get<1>().to_number(), match.get<2>().to_string());
    }

    if (auto match = ctre::search<calories_regex>(m_plaintext); match) {
      m_data.cals = match.get<0>().to_number() * m_amount;
    }
  }

  std::string initPlaintext(const std::string& query) {
    const auto WOLFRAM_KEY = utl::getEnv("NUTRI_WA_KEY");

    // clang-format off
    cpr::Response response = Get(
    cpr::Url{"https://api.wolframalpha.com/v2/query"},
    cpr::Parameters{
      {"input", query + "100g nutrition"},
      {"appid", WOLFRAM_KEY},
      {"output", "JSON"},
      {"format", "plaintext"}});
    // clang-format on

    glz::json_t json{};
    auto        ec = glz::read_json(json, response.text);
    if (ec == glz::error_code::none && json["queryresult"]["success"].get_boolean()) {
      return json["queryresult"]["pods"][1]["subpods"][0]["plaintext"].get<std::string>();
    } else {
      throw std::invalid_argument("WolframAlpha query failed");
    }
  }

  Data        m_data{};
  std::string m_plaintext{};
  double      m_amount{};
};
